package ajedrez.server.dominio;

import java.util.Vector;

import ajedrez.server.persistencia.Metodos;


/* Controla las funciones del ajedrez real. */
public class Tablero {
	private int nId = -1;  //Identificador de la partida
	private String sJugador1 = "-";  //Jugador de blancas
	private String sJugador2 = "-";  //Jugador de negras
	private int nGanador = -1;
		//0=tablas, 1=jugador1, 2=jugador2
	private Pieza [][] aapTablero = new Pieza[8][8];
	private Vector<String> vMovimientos = new Vector<String>();
	private String sMovimientos = "";
	private char cTurnoJugador = 'b';
	private Pieza [] apBlancas = new Pieza[16];
	private Pieza [] apNegras = new Pieza[16];
		// Ordenacin de las piezas: primero la fila de los peones y despus 
		//las dems piezas; por tanto, los peones van del 0 al 7, y el resto son:
	protected final int fnTorre1 = 8;
	protected final int fnTorre2 = 9;
	protected final int fnCaballo1 = 10;
	protected final int fnCaballo2 = 11;
	protected final int fnAlfil1 = 12;
	protected final int fnAlfil2 = 13;
	protected final int fnDama = 14;
	protected final int fnRey = 15;
	
	
	public Tablero(int nId, String sJug1, String sJug2) {
		this.nId = nId;
		sJugador1 = sJug1;
		sJugador2 = sJug2;
		nuevaPartida();
	} /* Fin mtodo constructor */
	
	
	/**
	 * Devuelve el nombre del jugador.
	 */
	public String getJugador(char cColor) {
		if (cColor == 'b') {
			return sJugador1;
		}
		else {
			return sJugador2;
		}
	} /* Fin mtodo getJugador */
	
	
	/**
	 * Devuelve el ganador de la partida.
	 */
	public int getGanador() {
		return nGanador;
	} /* Fin mtodo getGanador */
	
	
	/**
	 * Fija el ganador de la partida.
	 */
	protected void setGanador(char cColor) {
		if (cColor == 'b') {
			nGanador = 1;
		}
		else if (cColor == 'n') {
			nGanador = 2;
		}
		else {  //Tablas
			nGanador = 0;
		}
	} /* Fin mtodo setGanador */
	
	
	/**
	 * El jugador de las piezas de color 'cColor' ha abandonado.
	 */
	public void abandono(char cColor) {
		if (cColor == 'b') {
			setGanador('n');
		}
		else if (cColor == 'n') {
			setGanador('b');
		}
	} /* Fin mtodo abandono */
	
	
	/**
	 * Devuelve el tablero.
	 */
	public Pieza[][] getTablero() {
		return aapTablero;
	} /* Fin mtodo getTablero */
	
	
	/**
	 * Devuelve los movimientos realizados en forma de Vector.
	 */
	public Vector getMovimientosV() {
		return vMovimientos;
	} /* Fin mtodo getMovimientosV */
	
	
	/**
	 * Devuelve los movimientos realizados en forma de cadena.
	 */
	public String getMovimientosS() {
		return sMovimientos;
	} /* Fin mtodo getMovimientosS */
	
	
	/**
	 * Devuelve el color del jugador que tiene el turno.
	 */
	public char getTurno() {
		return cTurnoJugador;
	} /* Fin mtodo getTurno */
	
	
	/**
	 * Fija el color del jugador que tiene el turno.
	 */
	public void setTurno(char cTurno) {
		cTurnoJugador = cTurno;
	} /* Fin mtodo setTurno */
	
	
	/**
	 * Devuelve el color del jugador que no tiene el turno.
	 */
	public char getTurnoRival() {
		if (getTurno() == 'b') {
			return 'n';
		}
		else {
			return 'b';
		}
	} /* Fin mtodo getTurnoRival */
	
	
	/**
	 * Devuelve los enlaces a las piezas blancas.
	 */
	public Pieza[] getBlancas() {
		return apBlancas;
	} /* Fin mtodo getBlancas */
	
	
	/**
	 * Devuelve los enlaces a las piezas negras.
	 */
	public Pieza[] getNegras() {
		return apNegras;
	} /* Fin mtodo getNegras */
	
	
	/**
	 * Devuelve la fila que ocupa el rey indicado.
	 */
	public int getFilaRey(char cColor) {
		if (cColor == 'b') {
			return apBlancas[fnRey].getFila();
		}
		else {
			return apNegras[fnRey].getFila();
		}
	} /* Fin mtodo getFilaRey */
	
	
	/**
	 * Devuelve la columna que ocupa el rey indicado.
	 */
	public int getColumnaRey(char cColor) {
		if (cColor=='b') {
			return apBlancas[fnRey].getColumna();
		}
		else {
			return apNegras[fnRey].getColumna();
		}
	} /* Fin mtodo getColumnaRey */
	
	
	/**
	 * Crea las piezas en sus posiciones originales.
	 */
	private void crearPiezas() {
		//Creacin de las piezas blancas
		for (int i=0; i<8; i++) {
			apBlancas[i] = new Peon(i, 'b', 1, i);
		}
		apBlancas[fnTorre1] = new Torre(fnTorre1, 'b', 0, 0);
		apBlancas[fnCaballo1] = new Caballo(fnCaballo1, 'b', 0, 1);
		apBlancas[fnAlfil1] = new Alfil(fnAlfil1, 'b', 0, 2);
		apBlancas[fnRey] = new Rey(fnRey, 'b', 0, 3);
		apBlancas[fnDama] = new Dama(fnDama, 'b', 0, 4);
		apBlancas[fnAlfil2] = new Alfil(fnAlfil2, 'b', 0, 5);
		apBlancas[fnCaballo2] = new Caballo(fnCaballo2, 'b', 0, 6);
		apBlancas[fnTorre2] = new Torre(fnTorre2, 'b', 0, 7);
		
		//Creacin de las piezas negras
		for (int i=0; i<8; i++) {
			apNegras[i] = new Peon(i, 'n', 6, i);
		}
		apNegras[fnTorre1] = new Torre(fnTorre1, 'n', 7, 0);
		apNegras[fnCaballo1] = new Caballo(fnCaballo1, 'n', 7, 1);
		apNegras[fnAlfil1] = new Alfil(fnAlfil1, 'n', 7, 2);
		apNegras[fnRey] = new Rey(fnRey, 'n', 7, 3);
		apNegras[fnDama] = new Dama(fnDama, 'n', 7, 4);
		apNegras[fnAlfil2] = new Alfil(fnAlfil2, 'n', 7, 5);
		apNegras[fnCaballo2] = new Caballo(fnCaballo2, 'n', 7, 6);
		apNegras[fnTorre2] = new Torre(fnTorre2, 'n', 7, 7);
	} /* Fin mtodo crearPiezas */
	
	
	/**
	 * Crea una nueva partida, inicializando el tablero.
	 */
	protected void nuevaPartida() {
		// Se crean las piezas
		crearPiezas();
		// Se limpia el tablero
		for (int i=0; i<8; i++) {
			for (int j=0; j<8; j++) {
				aapTablero[i][j] = null;
			}
		}
		
		// Disposicin de las fichas blancas
		aapTablero[apBlancas[fnTorre1].getFila()][apBlancas[fnTorre1].getColumna()] = apBlancas[fnTorre1];
		aapTablero[apBlancas[fnCaballo1].getFila()][apBlancas[fnCaballo1].getColumna()] = apBlancas[fnCaballo1];
		aapTablero[apBlancas[fnAlfil1].getFila()][apBlancas[fnAlfil1].getColumna()] = apBlancas[fnAlfil1];
		aapTablero[apBlancas[fnRey].getFila()][apBlancas[fnRey].getColumna()] = apBlancas[fnRey];
		aapTablero[apBlancas[fnDama].getFila()][apBlancas[fnDama].getColumna()] = apBlancas[fnDama];
		aapTablero[apBlancas[fnAlfil2].getFila()][apBlancas[fnAlfil2].getColumna()] = apBlancas[fnAlfil2];
		aapTablero[apBlancas[fnCaballo2].getFila()][apBlancas[fnCaballo2].getColumna()] = apBlancas[fnCaballo2];
		aapTablero[apBlancas[fnTorre2].getFila()][apBlancas[fnTorre2].getColumna()] = apBlancas[fnTorre2];
		for (int i=0; i<8; i++) {
			aapTablero[apBlancas[i].getFila()][apBlancas[i].getColumna()] = apBlancas[i];
		}
		
		// Disposicin de las fichas negras
		for (int i=0; i<8; i++) {
			aapTablero[apNegras[i].getFila()][apNegras[i].getColumna()] = apNegras[i];
		}
		aapTablero[apNegras[fnTorre1].getFila()][apNegras[fnTorre1].getColumna()] = apNegras[fnTorre1];
		aapTablero[apNegras[fnCaballo1].getFila()][apNegras[fnCaballo1].getColumna()] = apNegras[fnCaballo1];
		aapTablero[apNegras[fnAlfil1].getFila()][apNegras[fnAlfil1].getColumna()] = apNegras[fnAlfil1];
		aapTablero[apNegras[fnRey].getFila()][apNegras[fnRey].getColumna()] = apNegras[fnRey];
		aapTablero[apNegras[fnDama].getFila()][apNegras[fnDama].getColumna()] = apNegras[fnDama];
		aapTablero[apNegras[fnAlfil2].getFila()][apNegras[fnAlfil2].getColumna()] = apNegras[fnAlfil2];
		aapTablero[apNegras[fnCaballo2].getFila()][apNegras[fnCaballo2].getColumna()] = apNegras[fnCaballo2];
		aapTablero[apNegras[fnTorre2].getFila()][apNegras[fnTorre2].getColumna()] = apNegras[fnTorre2];
	} /* Fin mtodo nuevaPartida */
	
	
	/**
	 * Gestiona la peticin de un movimiento.
	 * @return   Clave indicativa del suceso producido. Puede ser:
	 *         0: el movimiento se ha realizado correctamente
	 *         1: estado 0 y el jugador actual ha eliminado una pieza del contrincante
	 *         2: estado 0 y la pieza es un pen que ha llegado al final del tablero
	 *         3: estados 1 y 2
	 *         4: estado 0 y el movimiento realizado ha sido tomar al paso
	 *         5: estado 0 y el movimiento realizado ha sido el enroque corto
	 *         6: estado 0 y el movimiento realizado ha sido el enroque largo
	 *        1_: estado menor de 10 y se produce jaque al rey contrario
	 *        2_: estado menor de 10 y el jugador actual ha ganado la partida
	 *        3_: estado menor de 10 y se han producido tablas
	 *        -1: no es el turno del jugador actual
	 *        -2: no hay ninguna pieza en el origen
	 *        -3: la pieza de la casilla origen no es del jugador actual
	 *        -4: hay una pieza del jugador actual en la casilla destino
	 *        -5: el movimiento no se corresponde con la pieza del origen
	 *        -6: hay una pieza en medio que impide el movimiento solicitado
	 *        -7: el movimiento dejara al rey en jaque
	 */
	public int mover(char cTurno, int nFilaOrigen, int nColumnaOrigen, int nFilaDestino, int nColumnaDestino, String sCasillaOrigen, String sCasillaDestino) {
		int nCaso = 0;
		Rey piezaRey;
		boolean bEsFin = false;
		//Acoplamiento de los valores a la nomenclatura interna
		nFilaOrigen--;
		nColumnaOrigen--;
		nFilaDestino--;
		nColumnaDestino--;
		if (cTurno != getTurno()) {  //Caso -1
			nCaso = -1;
		}
		else if (aapTablero[nFilaOrigen][nColumnaOrigen] == null) {  //Caso -2
			nCaso = -2;
		}
		else if (aapTablero[nFilaOrigen][nColumnaOrigen].getColor() != getTurno()) {  //Caso -3
			nCaso = -3;
		}
		else if ((aapTablero[nFilaDestino][nColumnaDestino] != null) &&
				(aapTablero[nFilaOrigen][nColumnaOrigen].getColor()==
				aapTablero[nFilaDestino][nColumnaDestino].getColor())) {  //Caso -4
			nCaso = -4;
		}
		else {
			nCaso = aapTablero[nFilaOrigen][nColumnaOrigen].esValido(aapTablero, nFilaOrigen, nColumnaOrigen, nFilaDestino, nColumnaDestino);
			if (nCaso == -1) {  //Movimiento no vlido (caso -5)
				nCaso = -5;
			}
			else if (nCaso == -2) {  //Movimiento no vlido (caso -6)
				nCaso = -6;
			}
			else if (quedaReyEnJaque(nFilaOrigen, nColumnaOrigen, nFilaDestino, nColumnaDestino)) {  //Caso -7
				nCaso = -7;
			}
			else {  //Movimiento en principio vlido (casos positivos o '0')
				if (nCaso == 1) {  //Caso 4: tomar al paso
					if (esValidoTomarAlPaso(nFilaOrigen, nColumnaOrigen, nFilaDestino, nColumnaDestino)) {
						nCaso = 4;
					}
					else {
						nCaso = -5;
					}
				}
				else if (nCaso == 2) {  //Caso 5: enroque corto
					if (!estaReyEnJaque(cTurno, nFilaOrigen, nColumnaOrigen) && 
							!quedaReyEnJaque(nFilaOrigen, nColumnaOrigen, nFilaDestino, nColumnaDestino+1)) {
						nCaso = 5;
					}
					else {
						nCaso = -5;
					}
				}
				else if (nCaso == 3) {  //Caso 6: enroque largo
					if (!estaReyEnJaque(cTurno, nFilaOrigen, nColumnaOrigen) && 
							!quedaReyEnJaque(nFilaOrigen, nColumnaOrigen, nFilaDestino, nColumnaDestino-1)) {
						nCaso = 6;
					}
					else {
						nCaso = -5;
					}
				}
				else if (aapTablero[nFilaDestino][nColumnaDestino] == null) {  //Caso 0
					nCaso = 0;
				}
				else {  //Caso 1
					nCaso = 1;
					if (getTurno() == 'b') {
						apNegras[aapTablero[nFilaDestino][nColumnaDestino].getId()] = null;
					}
					else {
						apBlancas[aapTablero[nFilaDestino][nColumnaDestino].getId()] = null;
					}
				}
				
				if (nCaso >= 0) {  //Movimiento seguro vlido (casos positivos o '0')
					//Realizacin del movimiento
					aapTablero[nFilaDestino][nColumnaDestino] = aapTablero[nFilaOrigen][nColumnaOrigen];
					aapTablero[nFilaOrigen][nColumnaOrigen] = null;
					aapTablero[nFilaDestino][nColumnaDestino].setPosicion(nFilaDestino,nColumnaDestino);
					if (nCaso == 4) {  //Caso 4: tomar al paso
						if (getTurno() == 'b') {
							apNegras[aapTablero[nFilaOrigen][nColumnaDestino].getId()] = null;
							aapTablero[nFilaOrigen][nColumnaDestino] = null;
						}
						else {
							apBlancas[aapTablero[nFilaOrigen][nColumnaDestino].getId()] = null;
							aapTablero[nFilaOrigen][nColumnaDestino] = null;
						}
					}
					else if (nCaso == 5) {  //Caso 5: enroque corto
						aapTablero[nFilaDestino][nColumnaDestino+1] = aapTablero[nFilaOrigen][nColumnaOrigen-3];
						aapTablero[nFilaOrigen][nColumnaOrigen-3] = null;
						aapTablero[nFilaDestino][nColumnaDestino+1].setPosicion(nFilaDestino,nColumnaDestino+1);
					}
					else if (nCaso == 6) {  //Caso 6: enroque largo
						aapTablero[nFilaDestino][nColumnaDestino-1] = aapTablero[nFilaOrigen][nColumnaOrigen+4];
						aapTablero[nFilaOrigen][nColumnaOrigen+4] = null;
						aapTablero[nFilaDestino][nColumnaDestino-1].setPosicion(nFilaDestino,nColumnaDestino-1);
					}
					
					if (getTurno() == 'b') {
						piezaRey = (Rey)apBlancas[fnRey];
					}
					else {
						piezaRey = (Rey)apNegras[fnRey];
					}
					//Comprobacin si se deshabilita el enroque corto
					if (aapTablero[nFilaDestino][nColumnaDestino].getTipo().equals("t") && nColumnaOrigen==0) {
						piezaRey.deshabilitarEnroqueCorto();
					}
					//Comprobacin si se deshabilita el enroque largo
					else if (aapTablero[nFilaDestino][nColumnaDestino].getTipo().equals("t") && nColumnaOrigen==7) {
						piezaRey.deshabilitarEnroqueLargo();
					}
					//Comprobacin si se deshabilitan los dos enroques
					else if (aapTablero[nFilaDestino][nColumnaDestino].getTipo().equals("r")) {
						piezaRey.deshabilitarEnroqueCorto();
						piezaRey.deshabilitarEnroqueLargo();
					}
					//Comprobacin si la pieza es un pen que ha llegado al final del tablero
					if ((aapTablero[nFilaDestino][nColumnaDestino].getTipo().equals("p")) 
							&& (nFilaDestino==0 || nFilaDestino==7)) {  //Caso 2 o 3
						nCaso += 2;
					}
					//Comprobacin de jaque al rey contrario
					bEsFin = estaReyEnJaque(getTurnoRival(), getFilaRey(getTurnoRival()), getColumnaRey(getTurnoRival()));
					if (bEsFin) {  //Casos 1_ y 2_
						//Comprobacin de fin de partida con ganador y perdedor
						bEsFin = hayGanador(getTurno());
						if (bEsFin) {  //Caso 2_: es jaque-mate
							nCaso += 20;
							setGanador(cTurno);
						}
						else {  //Caso 1_: slo es jaque
							nCaso += 10;
						}
					}
					else {
						//Comprobacin de fin de partida en tablas
						bEsFin = hayTablas(sCasillaOrigen, sCasillaDestino);
						if (bEsFin) {  //Caso 3_
							nCaso += 30;
							setGanador('t');
						}
					}
					// Cambio del turno
					setTurno(getTurnoRival());
				} /* Fin if - movimiento seguro vlido */
			} /* Fin if - movimiento en principio vlido */
		}
		return nCaso;
	} /* Fin mtodo mover */
	
	
	/**
	 * Determina si el movimiento tomar al paso es definitivamente vlido.
	 * @return   Devuelve 'true' si el movimiento es vlido o 'false' si no.
	 */
	private boolean esValidoTomarAlPaso(int nFilaOrigen, int nColumnaOrigen, int nFilaDestino, int nColumnaDestino) {
		// Al invocar este mtodo las posiciones actuales de las piezas son 
		//las correctas, debiendo comprobar nicamente si el ltimo movimiento 
		//realizado fue la salida del pen contrario.
		boolean bEsValido = false;
		String sMovimientoRealizado = vMovimientos.get(vMovimientos.size()-1);
		String sColumnaOrigenAntigua = sMovimientoRealizado.substring(0,1);
		int nFilaOrigenAntigua = Integer.parseInt(sMovimientoRealizado.substring(1,2)) - 1;
		String sColumnaDestinoAntigua = sMovimientoRealizado.substring(3,4);
		int nFilaDestinoAntigua = Integer.parseInt(sMovimientoRealizado.substring(4,5)) - 1;
		String sColumnaDestino = "";
		switch (nColumnaDestino) {
			case 0:
				sColumnaDestino = "h";
				break;
			case 1:
				sColumnaDestino = "g";
				break;
			case 2:
				sColumnaDestino = "f";
				break;
			case 3:
				sColumnaDestino = "e";
				break;
			case 4:
				sColumnaDestino = "d";
				break;
			case 5:
				sColumnaDestino = "c";
				break;
			case 6:
				sColumnaDestino = "b";
				break;
			case 7:
				sColumnaDestino = "a";
				break;
		}
		
		/* Comparacin de la columna */
		// Deben ser iguales las dos antiguas y adems la destino actual
		if (sColumnaOrigenAntigua.equals(sColumnaDestinoAntigua) && sColumnaOrigenAntigua.equals(sColumnaDestino)) {
			bEsValido = true;
			
			/* Comparacin de la fila */
			// Las filas antiguas deben ser +1 y -1 de la actual
			if (((aapTablero[nFilaOrigen][nColumnaOrigen].getColor()=='b')
					&& nFilaOrigenAntigua==nFilaDestino+1 
					&& nFilaDestinoAntigua==nFilaDestino-1)
					|| ((aapTablero[nFilaOrigen][nColumnaOrigen].getColor()=='n')
					&& nFilaOrigenAntigua==nFilaDestino-1 
					&& nFilaDestinoAntigua==nFilaDestino+1)) {
				bEsValido = true;
			}
			else {
				bEsValido = false;
			}
		}
		return bEsValido;
	} /* Fin mtodo esValidoTomarAlPaso */
	
	
	/**
	 * Comprueba si, tras el movimiento, el rey queda en jaque.
	 * @return   'true' si el rey queda en jaque o 'false' si no.
	 */
	protected boolean quedaReyEnJaque(int nFilaOrigen, int nColumnaOrigen, int nFilaDestino, int nColumnaDestino) {
		boolean bJaque = false;
		char cTurno = aapTablero[nFilaOrigen][nColumnaOrigen].getColor();
		Pieza pCasillaDestino = aapTablero[nFilaDestino][nColumnaDestino];
		//Se simula el movimiento
		aapTablero[nFilaDestino][nColumnaDestino] = aapTablero[nFilaOrigen][nColumnaOrigen];
		aapTablero[nFilaOrigen][nColumnaOrigen] = null;
		//Se comprueba si hay jaque
		if (aapTablero[nFilaDestino][nColumnaDestino].getTipo().equals("r")) {
			bJaque = estaReyEnJaque(cTurno, nFilaDestino, nColumnaDestino);
		}
		else {
			bJaque = estaReyEnJaque(cTurno, getFilaRey(cTurno), getColumnaRey(cTurno));
		}
		//Se deshace el movimiento
		aapTablero[nFilaOrigen][nColumnaOrigen] = aapTablero[nFilaDestino][nColumnaDestino];
		aapTablero[nFilaDestino][nColumnaDestino] = pCasillaDestino;
		return bJaque;
	} /* Fin mtodo quedaReyEnJaque */
	
	
	/**
	 * Comprueba si el rey est en jaque.
	 * @return   'true' si el rey est en jaque o 'false' si no.
	 */
	protected boolean estaReyEnJaque(char cColor, int nFila, int nColumna) {
		boolean bJaque = false;
		if (indagarJaque(cColor, nFila, nColumna, new int[2]) > 0) {
			bJaque = true;
		}
		else {  // indagarJaque(cColor, nFila, nColumna, new int[]) == 0
			bJaque = false;
		}
		return bJaque;
	} /* Fin mtodo estaReyEnJaque */
	
	
	/**
	 * Comprueba si el jugador actual ha ganado la partida.
	 * @return   'true' si el jugador ha ganado o 'false' si no.
	 */
	protected boolean hayGanador(char cTurno) {
		boolean bHayGanador = true;
		int nFilaRey = -1;  // Fila que ocupa el rey
		int nColumnaRey = -1;  // Columna que ocupa el rey
		char cTurnoRival;
		int nNumeroJaques = 0;  // Nmero de jaques que recibe el rey
		int [] anPosJaque = new int[2];  // Posicin de la nica pieza que da jaque al rey
		if (cTurno == 'b') {
			cTurnoRival = 'n';
		}
		else {
			cTurnoRival = 'b';
		}
		nFilaRey = getFilaRey(cTurnoRival);
		nColumnaRey = getColumnaRey(cTurnoRival);
		
		/* Se valoran cada una de las nueve posiciones posibles del rey. */
		if (bHayGanador) {  //Arriba-Izquierda
			if (nFilaRey-1>=0 && nColumnaRey-1>=0 && 
					(aapTablero[nFilaRey-1][nColumnaRey-1]==null || 
					aapTablero[nFilaRey-1][nColumnaRey-1].getColor()==cTurno)) {
				bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey-1,nColumnaRey-1);
			}
			
			if (bHayGanador) {  //Arriba-Centro
				if (nFilaRey-1>=0 && 
						(aapTablero[nFilaRey-1][nColumnaRey]==null || 
						aapTablero[nFilaRey-1][nColumnaRey].getColor()==cTurno)) {
					bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey-1,nColumnaRey);
				}
				
				if (bHayGanador) {  //Arriba-Derecha
					if (nFilaRey-1>=0 && nColumnaRey+1<=7 && 
							(aapTablero[nFilaRey-1][nColumnaRey+1]==null || 
							aapTablero[nFilaRey-1][nColumnaRey+1].getColor()==cTurno)) {
						bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey-1,nColumnaRey+1);
					}
					
					if (bHayGanador) {  //Centro-Izquierda
						if (nColumnaRey-1>=0 && 
								(aapTablero[nFilaRey][nColumnaRey-1]==null || 
								aapTablero[nFilaRey][nColumnaRey-1].getColor()==cTurno)) {
							bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey,nColumnaRey-1);
						}
						
						if (bHayGanador) {  //Centro-Centro
							if (cTurno=='b') {
								bHayGanador = estaReyEnJaque('n',nFilaRey,nColumnaRey);
							}
							else {
								bHayGanador = estaReyEnJaque('b',nFilaRey,nColumnaRey);
							}
							
							if (bHayGanador) {  //Centro-Derecha
								if (nColumnaRey+1<=7 && 
										(aapTablero[nFilaRey][nColumnaRey+1]==null || 
										aapTablero[nFilaRey][nColumnaRey+1].getColor()==cTurno)) {
									bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey,nColumnaRey+1);
								}
								
								if (bHayGanador) {  //Abajo-Izquierda
									if (nFilaRey+1<=7 && nColumnaRey-1>=0 && 
											(aapTablero[nFilaRey+1][nColumnaRey-1]==null || 
											aapTablero[nFilaRey+1][nColumnaRey-1].getColor()==cTurno)) {
										bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey+1,nColumnaRey-1);
									}
									
									if (bHayGanador) {  //Abajo-Centro
										if (nFilaRey+1<=7 && 
												(aapTablero[nFilaRey+1][nColumnaRey]==null || 
												aapTablero[nFilaRey+1][nColumnaRey].getColor()==cTurno)) {
											bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey+1,nColumnaRey);
										}
										
										if (bHayGanador) {  //Abajo-Derecha
											if (nFilaRey+1<=7 && nColumnaRey+1<=7 && 
													(aapTablero[nFilaRey+1][nColumnaRey+1]==null || 
													aapTablero[nFilaRey+1][nColumnaRey+1].getColor()==cTurno)) {
												bHayGanador = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey+1,nColumnaRey+1);
											}
										}  //Abajo-Derecha
									}  //Abajo-Centro
								}  //Abajo-Izquierda
							}  //Centro-Derecha
						}  //Centro-Centro
					}  //Centro-Izquierda
				}  //Arriba-Derecha
			}  //Arriba-Centro
		}  //Arriba-Izquierda
		
		/* Si el rey slo recibe jaque de una pieza contraria, pero no tiene
		 escapatoria, puede ser que una pieza compaera suya corte el jaque. */
		nNumeroJaques = indagarJaque(cTurnoRival, nFilaRey, nColumnaRey, anPosJaque);
		if (nNumeroJaques==1 && bHayGanador) {
			if (hayPiezaQueSalveRey(cTurnoRival, nFilaRey, nColumnaRey, anPosJaque[0], anPosJaque[1])) {
				bHayGanador = false;
			}
			else {
				bHayGanador = true;
			}
		}
		
		return bHayGanador;
	} /* Fin mtodo hayGanador */
	
	
	/**
	 * Contabiliza el nmero de jaques que recibe el rey, guardando las
	 * posiciones de las piezas que dan jaques.
	 * @param cColor      Color del rey
	 * @param nFila       Fila que ocupa el rey
	 * @param nColumna    Columna que ocupa el rey
	 * @param anPosJaque  Parmetro de salida que, en el caso de atacar slo
	 *                    una pieza al rey, devuelve su posicin
	 * @return   Nmero de jaques que recibe el rey.
	 */
	private int indagarJaque(char cColor, int nFila, int nColumna, int [] anPosJaque) {
		int nNumeroJaques = 0;
		int nFilaRey = -1;
		int nColumnaRey = -1;
		boolean bJaque = false;
		if (nNumeroJaques < 2) {  //Arriba-Izquierda
			nFilaRey = nFila;
			nColumnaRey = nColumna;
			bJaque = false;
			while (!bJaque && nFilaRey>0 && nColumnaRey>0) {
				nFilaRey--;
				nColumnaRey--;
				if (aapTablero[nFilaRey][nColumnaRey]!=null) {
					if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
							(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("a")) || 
							(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
							(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
							nFila-1==nFilaRey && nColumna-1==nColumnaRey) || 
							(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("p") && 
							aapTablero[nFilaRey][nColumnaRey].getColor()=='b' && 
							nFila-1==nFilaRey && nColumna-1==nColumnaRey))) {
						nNumeroJaques++;
						anPosJaque[0] = nFilaRey;
						anPosJaque[1] = nColumnaRey;
						bJaque = true;
					}
					else {
						nFilaRey = -1;
						nColumnaRey = -1;
					}
				}
			}
			
			if (nNumeroJaques < 2) {  //Arriba-Centro
				nFilaRey = nFila;
				nColumnaRey = nColumna;
				bJaque = false;
				while (!bJaque && nFilaRey>0) {
					nFilaRey--;
					if (aapTablero[nFilaRey][nColumnaRey]!=null) {
						if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
								(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("t")) || 
								(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
								(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
								nFila-1==nFilaRey))) {
							nNumeroJaques++;
							anPosJaque[0] = nFilaRey;
							anPosJaque[1] = nColumnaRey;
							bJaque = true;
						}
						else {
							nFilaRey = -1;
						}
					}
				}
				
				if (nNumeroJaques < 2) {  //Arriba-Derecha
					nFilaRey = nFila;
					nColumnaRey = nColumna;
					bJaque = false;
					while (!bJaque && nFilaRey>0 && nColumnaRey<7) {
						nFilaRey--;
						nColumnaRey++;
						if (aapTablero[nFilaRey][nColumnaRey]!=null) {
							if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
									(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("a")) || 
									(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
									(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
									nFila-1==nFilaRey && nColumna+1==nColumnaRey) || 
									(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("p") && 
									aapTablero[nFilaRey][nColumnaRey].getColor()=='b' && 
									nFila-1==nFilaRey && nColumna+1==nColumnaRey))) {
								nNumeroJaques++;
								anPosJaque[0] = nFilaRey;
								anPosJaque[1] = nColumnaRey;
								bJaque = true;
							}
							else {
								nFilaRey = -1;
								nColumnaRey = 8;
							}
						}
					}
					
					if (nNumeroJaques < 2) {  //Centro-Izquierda
						nFilaRey = nFila;
						nColumnaRey = nColumna;
						bJaque = false;
						while (!bJaque && nColumnaRey>0) {
							nColumnaRey--;
							if (aapTablero[nFilaRey][nColumnaRey]!=null) {
								if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
										(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("t")) || 
										(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
										(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
										nColumna-1==nColumnaRey))) {
									nNumeroJaques++;
									anPosJaque[0] = nFilaRey;
									anPosJaque[1] = nColumnaRey;
									bJaque = true;
								}
								else {
									nColumnaRey = -1;
								}
							}
						}
						
						if (nNumeroJaques < 2) {  //Centro-Derecha
							nFilaRey = nFila;
							nColumnaRey = nColumna;
							bJaque = false;
							while (!bJaque && nColumnaRey<7) {
								nColumnaRey++;
								if (aapTablero[nFilaRey][nColumnaRey]!=null) {
									if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
											(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("t")) || 
											(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
											(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
											nColumna+1==nColumnaRey))) {
										nNumeroJaques++;
										anPosJaque[0] = nFilaRey;
										anPosJaque[1] = nColumnaRey;
										bJaque = true;
									}
									else {
										nColumnaRey = 8;
									}
								}
							}
							
							if (nNumeroJaques < 2) {  //Abajo-Izquierda
								nFilaRey = nFila;
								nColumnaRey = nColumna;
								bJaque = false;
								while (!bJaque && nFilaRey<7 && nColumnaRey>0) {
									nFilaRey++;
									nColumnaRey--;
									if (aapTablero[nFilaRey][nColumnaRey]!=null) {
										if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
												(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("a")) || 
												(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
												(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
												nFila+1==nFilaRey && nColumna-1==nColumnaRey) || 
												(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("p") && 
												aapTablero[nFilaRey][nColumnaRey].getColor()=='n' && 
												nFila+1==nFilaRey && nColumna-1==nColumnaRey))) {
											nNumeroJaques++;
											anPosJaque[0] = nFilaRey;
											anPosJaque[1] = nColumnaRey;
											bJaque = true;
										}
										else {
											nFilaRey = 8;
											nColumnaRey = -1;
										}
									}
								}
								
								if (nNumeroJaques < 2) {  //Abajo-Centro
									nFilaRey = nFila;
									nColumnaRey = nColumna;
									bJaque = false;
									while (!bJaque && nFilaRey<7) {
										nFilaRey++;
										if (aapTablero[nFilaRey][nColumnaRey]!=null) {
											if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
													(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("t")) || 
													(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
													(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
													nFila+1==nFilaRey))) {
												nNumeroJaques++;
												anPosJaque[0] = nFilaRey;
												anPosJaque[1] = nColumnaRey;
												bJaque = true;
											}
											else {
												nFilaRey = 8;
											}
										}
									}
									
									if (nNumeroJaques < 2) {  //Abajo-Derecha
										nFilaRey = nFila;
										nColumnaRey = nColumna;
										bJaque = false;
										while (!bJaque && nFilaRey<7 && nColumnaRey<7) {
											nFilaRey++;
											nColumnaRey++;
											if (aapTablero[nFilaRey][nColumnaRey]!=null) {
												if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
														(((aapTablero[nFilaRey][nColumnaRey].getTipo().equals("a")) || 
														(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("d"))) || 
														(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("r") && 
														nFila+1==nFilaRey && nColumna+1==nColumnaRey) || 
														(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("p") && 
														aapTablero[nFilaRey][nColumnaRey].getColor()=='n' && 
														nFila+1==nFilaRey && nColumna+1==nColumnaRey))) {
													nNumeroJaques++;
													anPosJaque[0] = nFilaRey;
													anPosJaque[1] = nColumnaRey;
													bJaque = true;
												}
												else {
													nFilaRey = 8;
													nColumnaRey = 8;
												}
											}
										}
										
										if (nNumeroJaques < 2) {  //Caballo en Arriba-Arriba-Izquierda
											nFilaRey = nFila;
											nColumnaRey = nColumna;
											nFilaRey--;
											nFilaRey--;
											nColumnaRey--;
											if (nFilaRey>=0 && nColumnaRey>=0 && aapTablero[nFilaRey][nColumnaRey]!=null) {
												if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
														(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
													nNumeroJaques++;
													anPosJaque[0] = nFilaRey;
													anPosJaque[1] = nColumnaRey;
												}
											}
											
											if (nNumeroJaques < 2) {  //Caballo en Arriba-Arriba-Derecha
												nFilaRey = nFila;
												nColumnaRey = nColumna;
												nFilaRey--;
												nFilaRey--;
												nColumnaRey++;
												if (nFilaRey>=0 && nColumnaRey<=7 && aapTablero[nFilaRey][nColumnaRey]!=null) {
													if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
															(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
														nNumeroJaques++;
														anPosJaque[0] = nFilaRey;
														anPosJaque[1] = nColumnaRey;
													}
												}
												
												if (nNumeroJaques < 2) {  //Caballo en Derecha-Derecha-Arriba
													nFilaRey = nFila;
													nColumnaRey = nColumna;
													nFilaRey--;
													nColumnaRey++;
													nColumnaRey++;
													if (nFilaRey>=0 && nColumnaRey<=7 && aapTablero[nFilaRey][nColumnaRey]!=null) {
														if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
																(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
															nNumeroJaques++;
															anPosJaque[0] = nFilaRey;
															anPosJaque[1] = nColumnaRey;
														}
													}
													
													if (nNumeroJaques < 2) {  //Caballo en Derecha-Derecha-Abajo
														nFilaRey = nFila;
														nColumnaRey = nColumna;
														nFilaRey++;
														nColumnaRey++;
														nColumnaRey++;
														if (nFilaRey<=7 && nColumnaRey<=7 && aapTablero[nFilaRey][nColumnaRey]!=null) {
															if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
																	(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
																nNumeroJaques++;
																anPosJaque[0] = nFilaRey;
																anPosJaque[1] = nColumnaRey;
															}
														}
														
														if (nNumeroJaques < 2) {  //Caballo en Abajo-Abajo-Izquierda
															nFilaRey = nFila;
															nColumnaRey = nColumna;
															nFilaRey++;
															nFilaRey++;
															nColumnaRey--;
															if (nFilaRey<=7 && nColumnaRey>=0 && aapTablero[nFilaRey][nColumnaRey]!=null) {
																if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
																		(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
																	nNumeroJaques++;
																	anPosJaque[0] = nFilaRey;
																	anPosJaque[1] = nColumnaRey;
																}
															}
															
															if (nNumeroJaques < 2) {  //Caballo en Abajo-Abajo-Derecha
																nFilaRey = nFila;
																nColumnaRey = nColumna;
																nFilaRey++;
																nFilaRey++;
																nColumnaRey++;
																if (nFilaRey<=7 && nColumnaRey<=7 && aapTablero[nFilaRey][nColumnaRey]!=null) {
																	if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
																			(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
																		nNumeroJaques++;
																		anPosJaque[0] = nFilaRey;
																		anPosJaque[1] = nColumnaRey;
																	}
																}
																
																if (nNumeroJaques < 2) {  //Caballo en Izquierda-Izquierda-Arriba
																	nFilaRey = nFila;
																	nColumnaRey = nColumna;
																	nFilaRey--;
																	nColumnaRey--;
																	nColumnaRey--;
																	if (nFilaRey>=0 && nColumnaRey>=0 && aapTablero[nFilaRey][nColumnaRey]!=null) {
																		if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
																				(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
																			nNumeroJaques++;
																			anPosJaque[0] = nFilaRey;
																			anPosJaque[1] = nColumnaRey;
																		}
																	}
																	
																	if (nNumeroJaques < 2) {  //Caballo en Izquierda-Izquierda-Abajo
																		nFilaRey = nFila;
																		nColumnaRey = nColumna;
																		nFilaRey++;
																		nColumnaRey--;
																		nColumnaRey--;
																		if (nFilaRey<=7 && nColumnaRey>=0 && aapTablero[nFilaRey][nColumnaRey]!=null) {
																			if ((aapTablero[nFilaRey][nColumnaRey].getColor()!=cColor) && 
																					(aapTablero[nFilaRey][nColumnaRey].getTipo().equals("c"))) {
																				nNumeroJaques++;
																				anPosJaque[0] = nFilaRey;
																				anPosJaque[1] = nColumnaRey;
																			}
																		}
																	}  //Caballo en Izquierda-Izquierda-Abajo
																}  //Caballo en Izquierda-Izquierda-Arriba
															}  //Caballo en Abajo-Abajo-Derecha
														}  //Caballo en Abajo-Abajo-Izquierda
													}  //Caballo en Derecha-Derecha-Abajo
												}  //Caballo en Derecha-Derecha-Arriba
											}  //Caballo en Arriba-Arriba-Derecha
										}  //Caballo en Arriba-Arriba-Izquierda
									}  //Abajo-Derecha
								}  //Abajo-Centro
							}  //Abajo-Izquierda
						}  //Centro-Derecha
					}  //Centro-Izquierda
				}  //Arriba-Derecha
			}  //Arriba-Centro
		}  //Arriba-Izquierda
		return nNumeroJaques;
	} /* Fin mtodo indagarJaque */
	
	
	/**
	 * Determina si existe alguna pieza compaera del rey que pueda acabar con
	 * el nico jaque que ataca al rey, bien interponindose en medio, bien
	 * eliminando a la pieza que da el jaque.
	 * @param cColor         Color del rey
	 * @param nFilaRey       Fila que ocupa el rey
	 * @param nColumnaRey    Columna que ocupa el rey
	 * @param nFilaJaque     Fila que ocupa la nica pieza que da jaque al rey
	 * @param nColumnaJaque  Columna que ocupa la nica pieza que da jaque al rey
	 * @return   Devuelve 'true' si existe alguna pieza que pueda acabar con
	 *           el jaque, o 'false' si no.
	 */
	private boolean hayPiezaQueSalveRey(char cColor, int nFilaRey, int nColumnaRey, int nFilaJaque, int nColumnaJaque) {
		boolean bHaySalvador = false;
		Pieza [] apPiezas = null;
		if (cColor == 'b') {
			apPiezas = apBlancas;
		}
		else {
			apPiezas = apNegras;
		}
		//Para cada una de las piezas vivas menos el rey
		for (int i=0; !bHaySalvador && i<15; i++) {
			if (apPiezas[i] != null) {
				bHaySalvador = salvaRey(
						apPiezas[i].getFila(), apPiezas[i].getColumna(), 
						nFilaRey, nColumnaRey, 
						aapTablero[nFilaJaque][nColumnaJaque].getTipo(), 
						nFilaJaque, nColumnaJaque);
			}
		}
		return bHaySalvador;
	} /* Fin mtodo hayPiezaQueSalveRey */
	
	
	/**
	 * Investiga si la pieza, dependiendo de su tipo y posicin, puede cortar
	 * la lnea de jaque o comer al atacante.
	 * @param nFila          Fila de la posible pieza salvadora
	 * @param nColumna       Columna de la posible pieza salvadora
	 * @param nFilaRey       Fila que ocupa el rey
	 * @param nColumnaRey    Columna que ocupa el rey
	 * @param sAtacante      Tipo de la nica pieza que da jaque al rey
	 * @param nFilaJaque     Fila que ocupa la nica pieza que da jaque al rey
	 * @param nColumnaJaque  Columna que ocupa la nica pieza que da jaque al rey
	 * @return   Devuelve 'true' si la pieza puede cortar el jaque, y 'false'
	 *           en caso contrario.
	 */
	private boolean salvaRey(int nFila, int nColumna, int nFilaRey, int nColumnaRey, String sAtacante, int nFilaJaque, int nColumnaJaque) {
		boolean bEsSalvadora = false;
		// Opcin 1: la pieza salvadora come al atacante
		if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, nFilaJaque, nColumnaJaque)==0) {
			bEsSalvadora = true;
		}
		if (!bEsSalvadora) {
			// Opcin 2: la pieza salvadora corta el jaque
			if (sAtacante.equals("t")) {  // El atacante es una torre
				if (nFilaRey>nFilaJaque && nColumnaRey==nColumnaJaque) {  // Caso 1: Atacante al N del Rey
					for (int i=nFilaRey-1; !bEsSalvadora && i!=nFilaJaque; i--) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, nColumnaRey)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey==nFilaJaque && nColumnaRey<nColumnaJaque) {  // Caso 2: Atacante al E del Rey
					for (int j=nColumnaRey+1; !bEsSalvadora && j!=nColumnaJaque; j++) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, nFilaRey, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey<nFilaJaque && nColumnaRey==nColumnaJaque) {  // Caso 3: Atacante al S del Rey
					for (int i=nFilaRey+1; !bEsSalvadora && i!=nFilaJaque; i++) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, nColumnaRey)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey==nFilaJaque && nColumnaRey>nColumnaJaque) {  // Caso 4: Atacante al O del Rey
					for (int j=nColumnaRey-1; !bEsSalvadora && j!=nColumnaJaque; j--) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, nFilaRey, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
			} // Fin if - El atacante es una torre
			
			else if (sAtacante.equals("a")) {  // El atacante es un alfil
				if (nFilaRey>nFilaJaque && nColumnaRey<nColumnaJaque) {  // Caso 1: Atacante al NE del Rey
					for (int i=nFilaRey-1, j=nColumnaRey+1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i--, j++) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey<nFilaJaque && nColumnaRey<nColumnaJaque) {  // Caso 2: Atacante al SE del Rey
					for (int i=nFilaRey+1, j=nColumnaRey+1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i++, j++) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey<nFilaJaque && nColumnaRey>nColumnaJaque) {  // Caso 3: Atacante al SO del Rey
					for (int i=nFilaRey+1, j=nColumnaRey-1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i++, j--) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey>nFilaJaque && nColumnaRey>nColumnaJaque) {  // Caso 4: Atacante al NO del Rey
					for (int i=nFilaRey-1, j=nColumnaRey-1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i--, j--) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
			} // Fin if - El atacante es un alfil
			
			else if (sAtacante.equals("d")) {  // El atacante es una dama
				if (nFilaRey>nFilaJaque && nColumnaRey==nColumnaJaque) {  // Caso 1: Atacante al N del Rey
					for (int i=nFilaRey-1; !bEsSalvadora && i!=nFilaJaque; i--) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, nColumnaRey)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey==nFilaJaque && nColumnaRey<nColumnaJaque) {  // Caso 2: Atacante al E del Rey
					for (int j=nColumnaRey+1; !bEsSalvadora && j!=nColumnaJaque; j++) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, nFilaRey, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey<nFilaJaque && nColumnaRey==nColumnaJaque) {  // Caso 3: Atacante al S del Rey
					for (int i=nFilaRey+1; !bEsSalvadora && i!=nFilaJaque; i++) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, nColumnaRey)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else if (nFilaRey==nFilaJaque && nColumnaRey>nColumnaJaque) {  // Caso 4: Atacante al O del Rey
					for (int j=nColumnaRey-1; !bEsSalvadora && j!=nColumnaJaque; j--) {
						if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, nFilaRey, j)==0) {
							bEsSalvadora = true;
						}
					}
				}
				else {  // Ataca en una diagonal
					if (nFilaRey>nFilaJaque && nColumnaRey<nColumnaJaque) {  // Caso 5: Atacante al NE del Rey
						for (int i=nFilaRey-1, j=nColumnaRey+1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i--, j++) {
							if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
								bEsSalvadora = true;
							}
						}
					}
					else if (nFilaRey<nFilaJaque && nColumnaRey<nColumnaJaque) {  // Caso 6: Atacante al SE del Rey
						for (int i=nFilaRey+1, j=nColumnaRey+1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i++, j++) {
							if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
								bEsSalvadora = true;
							}
						}
					}
					else if (nFilaRey<nFilaJaque && nColumnaRey>nColumnaJaque) {  // Caso 7: Atacante al SO del Rey
						for (int i=nFilaRey+1, j=nColumnaRey-1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i++, j--) {
							if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
								bEsSalvadora = true;
							}
						}
					}
					else if (nFilaRey>nFilaJaque && nColumnaRey>nColumnaJaque) {  // Caso 8: Atacante al NO del Rey
						for (int i=nFilaRey-1, j=nColumnaRey-1; !bEsSalvadora && i!=nFilaJaque && j!=nColumnaJaque; i--, j--) {
							if (aapTablero[nFila][nColumna].esValido(aapTablero, nFila, nColumna, i, j)==0) {
								bEsSalvadora = true;
							}
						}
					}
				} // Fin if - Ataca en una diagonal
			} // Fin if - El atacante es una dama
			// Los jaques del caballo y el pen slo se pueden eliminar 
			//comiendo a la pieza atacante.
		}
		return bEsSalvadora;
	} /* Fin mtodo salvaRey */
	
	
	/**
	 * Comprueba si el resultado de la partida es tablas.
	 * @return   'true' si el resultado es tablas o 'false' si no.
	 */
	protected boolean hayTablas(String sCasillaOrigen, String sCasillaDestino) {
		boolean bHayTablas = true;
		String sMovimientoActual = sCasillaOrigen+"-"+sCasillaDestino;
		//Caso 1: slo quedan los reyes (piezas 0..14 = null)
		for (int i=0; bHayTablas && i<15; i++) {
			if (apBlancas[i]!=null || apNegras[i]!=null) {
				bHayTablas = false;
			}
		}
		if (!bHayTablas) {
			//Caso 2: jaque continuo
			int nTam = vMovimientos.size();
			if (nTam >= 11 && 
					sMovimientoActual.equals(vMovimientos.get(nTam-4)) && 
					vMovimientos.get(nTam-4).equals(vMovimientos.get(nTam-8)) && 
					vMovimientos.get(nTam-2).equals(vMovimientos.get(nTam-6)) && 
					vMovimientos.get(nTam-6).equals(vMovimientos.get(nTam-10)) && 
					vMovimientos.get(nTam-1).equals(vMovimientos.get(nTam-5)) && 
					vMovimientos.get(nTam-5).equals(vMovimientos.get(nTam-9)) && 
					vMovimientos.get(nTam-3).equals(vMovimientos.get(nTam-7)) && 
					vMovimientos.get(nTam-7).equals(vMovimientos.get(nTam-11))) {
				bHayTablas = true;
			}
		}
		if (!bHayTablas) {
			//Caso 3: mate ahogado
			if (estaJugadorAhogado('b') || estaJugadorAhogado('n')) {
				bHayTablas = true;
			}
		}
		return bHayTablas;
	} /* Fin mtodo hayTablas */
	
	
	/**
	 * Determina si un jugador est en "mate ahogado"; es decir, si no puede
	 * mover ninguna pieza, su rey no est en jaque y tras cualquier movimiento
	 * quedara en jaque.
	 * @return   'true' si el jugador est en mate ahogado y 'false' si no.
	 */
	private boolean estaJugadorAhogado(char cColor) {
		boolean bEstaAhogado = false;
		int nFilaRey = getFilaRey(cColor);
		int nColumnaRey = getColumnaRey(cColor);
		
		// El rey no debe estar en jaque
		if (!estaReyEnJaque(cColor, getFilaRey(cColor), getColumnaRey(cColor))) {
			bEstaAhogado = true;
			
			// Ninguna pieza se puede mover (el rey se comprueba despus)
			if (cColor == 'b') {
				for (int i=0; bEstaAhogado && i<15; i++) {
					if (apBlancas[i]!=null && !apBlancas[i].estaBloqueada(aapTablero)) {
						bEstaAhogado = false;
					}
				}
			}
			else {
				for (int i=0; bEstaAhogado && i<15; i++) {
					if (apNegras[i]!=null && !apNegras[i].estaBloqueada(aapTablero)) {
						bEstaAhogado = false;
					}
				}
			}
			
			// El rey debe estar en jaque al moverse
			if (bEstaAhogado) {  //Arriba-Izquierda
				if (nFilaRey-1>=0 && nColumnaRey-1>=0 && 
						(aapTablero[nFilaRey-1][nColumnaRey-1]==null || 
						aapTablero[nFilaRey-1][nColumnaRey-1].getColor()!=cColor)) {
					bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey-1,nColumnaRey-1);
				}
				
				if (bEstaAhogado) {  //Arriba-Centro
					if (nFilaRey-1>=0 && 
							(aapTablero[nFilaRey-1][nColumnaRey]==null || 
							aapTablero[nFilaRey-1][nColumnaRey].getColor()!=cColor)) {
						bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey-1,nColumnaRey);
					}
					
					if (bEstaAhogado) {  //Arriba-Derecha
						if (nFilaRey-1>=0 && nColumnaRey+1<=7 && 
								(aapTablero[nFilaRey-1][nColumnaRey+1]==null || 
								aapTablero[nFilaRey-1][nColumnaRey+1].getColor()!=cColor)) {
							bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey-1,nColumnaRey+1);
						}
						
						if (bEstaAhogado) {  //Centro-Izquierda
							if (nColumnaRey>=0 && 
									(aapTablero[nFilaRey][nColumnaRey-1]==null || 
									aapTablero[nFilaRey][nColumnaRey-1].getColor()!=cColor)) {
								bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey,nColumnaRey-1);
							}
							
							if (bEstaAhogado) {  //Centro-Derecha
								if (nColumnaRey+1<=7 && 
										(aapTablero[nFilaRey][nColumnaRey+1]==null || 
										aapTablero[nFilaRey][nColumnaRey+1].getColor()!=cColor)) {
									bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey,nColumnaRey+1);
								}
								
								if (bEstaAhogado) {  //Abajo-Izquierda
									if (nFilaRey+1<=7 && nColumnaRey-1>=0 && 
											(aapTablero[nFilaRey+1][nColumnaRey-1]==null || 
											aapTablero[nFilaRey+1][nColumnaRey-1].getColor()!=cColor)) {
										bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey+1,nColumnaRey-1);
									}
									
									if (bEstaAhogado) {  //Abajo-Centro
										if (nFilaRey+1<=7 && 
												(aapTablero[nFilaRey+1][nColumnaRey]==null || 
												aapTablero[nFilaRey+1][nColumnaRey].getColor()!=cColor)) {
											bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey+1,nColumnaRey);
										}
										
										if (bEstaAhogado) {  //Abajo-Derecha
											if (nFilaRey+1<=7 && nColumnaRey+1<=7 && 
													(aapTablero[nFilaRey+1][nColumnaRey+1]==null || 
													aapTablero[nFilaRey+1][nColumnaRey+1].getColor()!=cColor)) {
												bEstaAhogado = quedaReyEnJaque(nFilaRey,nColumnaRey,nFilaRey+1,nColumnaRey+1);
											}
										}  //Abajo-Derecha
									}  //Abajo-Centro
								}  //Abajo-Izquierda
							}  //Centro-Derecha
						}  //Centro-Izquierda
					}  //Arriba-Derecha
				}  //Arriba-Centro
			}  //Arriba-Izquierda
		}
		return bEstaAhogado;
	} /* Fin mtodo estaJugadorAhogado */
	
	
	/**
	 * Convierte el pen al tipo de pieza indicado.
	 * @return   Clave indicativa del suceso producido. Puede ser:
	 *         0: conversin realizada sin incidencias
	 *         1: estado 0 y se produce jaque al rey contrario
	 *         2: estado 0 y el jugador actual ha ganado la partida
	 *         3: estado 0 y se han producido tablas
	 */
	public int convertirPeon(char cTipo) {
		int nId=0, nFila=0, nColumna=0, nCaso=0;
		boolean bEsFin = false;
		Pieza nuevaPieza = null;
		// Bsqueda del pen
		if (getTurnoRival() == 'b') {  //(El turno ya se haba cambiado)
			for (int i=0; !bEsFin && i<8; i++) {
				if (apBlancas[i]!=null && apBlancas[i].getFila()==7) {
					bEsFin = true;
					nId = apBlancas[i].getId();
					nFila = apBlancas[i].getFila();
					nColumna = apBlancas[i].getColumna();
				}
			}
		}
		else {
			for (int i=0; !bEsFin && i<8; i++) {
				if (apNegras[i]!=null && apNegras[i].getFila()==0) {
					bEsFin = true;
					nId = apNegras[i].getId();
					nFila = apNegras[i].getFila();
					nColumna = apNegras[i].getColumna();
				}
			}
		}
		// Creacin de la nueva pieza
		switch (cTipo) {
			case 't':
				nuevaPieza = new Torre(nId, getTurnoRival(), nFila, nColumna);
				break;
			case 'c':
				nuevaPieza = new Caballo(nId, getTurnoRival(), nFila, nColumna);
				break;
			case 'a':
				nuevaPieza = new Alfil(nId, getTurnoRival(), nFila, nColumna);
				break;
			case 'd':
				nuevaPieza = new Dama(nId, getTurnoRival(), nFila, nColumna);
				break;
		}
		// Asignacin y colocacin de la nueva pieza en el tablero
		if (getTurnoRival() == 'b') {
			apBlancas[nId] = nuevaPieza;
		}
		else {
			apNegras[nId] = nuevaPieza;
		}
		aapTablero[nFila][nColumna] = nuevaPieza;
		
		//Comprobacin de jaque al rey contrario
		bEsFin = estaReyEnJaque(getTurno(), getFilaRey(getTurno()), getColumnaRey(getTurno()));
		if (bEsFin) {  //Casos 1 y 2
			nCaso = 1;
			//Comprobacin de fin de partida con ganador y perdedor
			bEsFin = hayGanador(getTurnoRival());
			if (bEsFin) {  //Caso 2
				nCaso = 2;
				setGanador(getTurnoRival());
			}
		}
		else {
			//Comprobacin de fin de partida en tablas
			bEsFin = hayTablas("","");  // Da igual el movimiento porque no se puede producir el jaque continuo
			if (bEsFin) {  //Caso 3
				nCaso = 3;
				setGanador('t');
			}
		}
		return nCaso;
	} /* Fin mtodo convertirPeon */
	
	
	/**
	 * Almacena el movimiento realizado.
	 */
	public void almacenarMovimiento(String sMov) {
		vMovimientos.add(sMov);
		sMovimientos += sMov + " ";
	} /* Fin mtodo almacenarMovimiento */
	
	
	/**
	 * Ordena el almacenamiento en la base de datos.
	 */
	public void guardarEnBD() {
		Metodos.insertaPartida(nId, sJugador1, sJugador2, nGanador, getMovimientosS());		
	} /* Fin mtodo guardarEnBD */
	
	
	/**
	 * Devuelve el tablero listo para mostrar por pantalla.
	 */
	public String toString() {
		String cadena = "\n\n";
		cadena += "DISPOSICION ACTUAL DEL TABLERO\n";
		cadena += "==============================\n\n";
		//Numeracin de las columnnas
		cadena += "\t ";
		for (int j=1; j<=8; j++) {
			cadena += " | " + j;
		}
		cadena += " |\n";
		//Inicio del tablero
		cadena += "\t--+---+---+---+---+---+---+---+---+\n";
		//Dibujo del tablero
		for (int i=0; i<8; i++) {
			if (i%2 != 0) {
				cadena += "\t" + "  |***|   |***|   |***|   |***|   |\n";
			}
			else {
				cadena += "\t" + "  |   |***|   |***|   |***|   |***|\n";
			}
			cadena += "\t" + (i+1) + " ";
			for (int j=0; j<8; j++) {
				cadena += "|";
				if ((i+j)%2 != 0) {  //Casilla negra
					cadena += "*";
					if (aapTablero[i][j] == null) {  //Casilla vaca
						cadena += "*";
					}
					else {  //Casilla con ficha
						if (aapTablero[i][j].getColor() == 'b') {  //Ficha blanca
							cadena += aapTablero[i][j].getTipo().toUpperCase();
						}
						else {  //Ficha negra
							cadena += aapTablero[i][j].getTipo().toLowerCase();
						}
					}
					cadena += "*";
				}
				else {  //Casilla blanca
					cadena += " ";
					if (aapTablero[i][j] == null) {  //Casilla vaca
						cadena += " ";
					}
					else {  //Casilla con ficha
						if (aapTablero[i][j].getColor() == 'b') {  //Ficha blanca
							cadena += aapTablero[i][j].getTipo().toUpperCase();
						}
						else {  //Ficha negra
							cadena += aapTablero[i][j].getTipo().toLowerCase();
						}
					}
					cadena += " ";
				}
			}
			cadena += "|\n";
			if (i%2 != 0) {
				cadena += "\t" + "  |***|   |***|   |***|   |***|   |\n";
			}
			else {
				cadena += "\t" + "  |   |***|   |***|   |***|   |***|\n";
			}
			if (i != 7) {
				cadena += "\t--+---+---+---+---+---+---+---+---+\n";
			}
		}
		//Final del tablero
		cadena += "\t--+---+---+---+---+---+---+---+---+\n";
		return cadena;
	} /* Fin mtodo toString */
	
} /* Fin clase Tablero */
