package ajedrez.server.persistencia;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Vector;

import ajedrez.server.dominio.Tablero;
import junit.framework.TestCase;

public class TestBaseDeDatos extends TestCase {

	String user1 = "test1";
	String user2 = "test2";
	String pass1 = "pass1";
	String pass2 = "pass2";
	
	Vector<String> partidaInserted;
	Vector<String> jugadorInserted;
	
	protected void setUp(){
		partidaInserted = new Vector<String>();
		jugadorInserted = new Vector<String>();
		PreparedStatement p = null;
		Connection bd = null;
		String SQL = "INSERT INTO Jugador (nombreUsuario, contrasea, [Partidas Jugadas], [Partidas Ganadas], [Partidas Perdidas], [Partidas Tablas], Puntuacion) values (?, ?, ?, ?, ?, ?, ?)";
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL);
			p.setString(1, user1);
			p.setString(2, pass1);
			p.setInt(3, 0);
			p.setInt(4, 0);
			p.setInt(5, 0);
			p.setInt(6, 0);
			p.setInt(7, 0);
			p.executeUpdate();
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL);
			p.setString(1, user2);
			p.setString(2, pass2);
			p.setInt(3, 0);
			p.setInt(4, 0);
			p.setInt(5, 0);
			p.setInt(6, 0);
			p.setInt(7, 0);
			p.executeUpdate();
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		
	}
	protected void tearDown(){
		PreparedStatement p = null;
		Connection bd = null;
		
		String SQL4 = "Delete from Jugador WHERE nombreUsuario = ?;";
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL4);
			p.setString(1, user1);
			p.executeUpdate();
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		
		try {
			bd = Agente.getAgente().getBD();
			p = bd.prepareStatement(SQL4);
			p.setString(1, user2);
			p.executeUpdate();
		}
		catch (SQLException eSQL) {
			eSQL.printStackTrace(); 
		}
		finally {
			try {
				bd.close();
			}
			catch (SQLException eSQL) {
				eSQL.printStackTrace();
			}
		}
		
		if(this.partidaInserted.size()>0){
			for(String part : partidaInserted){
				String SQL1 = "Delete from [Historial Partidas]  where [Id Partida] = ?;";
				try {
					bd = Agente.getAgente().getBD();
					p = bd.prepareStatement(SQL1);
					p.setString(1, part);
					p.executeUpdate();
				}
				catch (SQLException eSQL) {
					eSQL.printStackTrace(); 
				}
				finally {
					try {
						bd.close();
					}
					catch (SQLException eSQL) {
						eSQL.printStackTrace();
					}
				}
			}
			
			
		}
		
		if(this.jugadorInserted.size()>0){
			for(String jug:jugadorInserted){
				try {
					bd = Agente.getAgente().getBD();
					p = bd.prepareStatement(SQL4);
					p.setString(1, jug);
					p.executeUpdate();
				}
				catch (SQLException eSQL) {
					eSQL.printStackTrace(); 
				}
				finally {
					try {
						bd.close();
					}
					catch (SQLException eSQL) {
						eSQL.printStackTrace();
					}
				}
			}
			
		}
		
	}
	
	
	
	
	public void testGuardarEnBD() {
		
		
		
		int ultimoID = Metodos.getUltimoId();
		this.partidaInserted.add(""+(ultimoID+1));
		
		Tablero t = new Tablero(ultimoID+1, user1, user2);
		t.guardarEnBD();
		
		int ultimoID2 = Metodos.getUltimoId();
		
		assertTrue(ultimoID == ultimoID2-1);
		
	} 
	
	public void testCompruebaUsuario(){
		int r = Metodos.compruebaUsuario(user1, pass1);
		assertTrue(r == 0);
		
		r = Metodos.compruebaUsuario(user1, "novalida");
		assertTrue(r == -1);
		
		r = Metodos.compruebaUsuario("test3", pass1);
		assertTrue(r == -2);
		
	}
	
	public void testInsertarJugador(){
		this.jugadorInserted.add("test3");
		boolean  r = Metodos.insertaJugador("test3", "pass3");
		assertTrue(r);
		
		
	}
	
	public void testPuntuaciones(){
		int ultimoID = Metodos.getUltimoId();
		this.partidaInserted.add(""+(ultimoID+1));
		
		Tablero t = new Tablero(ultimoID+1, user1, user2);
		t.abandono('n');
		t.guardarEnBD();
		
		String [] datos = Metodos.getClasificacion(user1);
		assertTrue(datos[0].equals("1"));
		assertTrue(datos[1].equals("1"));
		assertTrue(datos[2].equals("0"));
		assertTrue(datos[3].equals("0"));
		assertTrue(datos[4].equals("3"));
		
		datos = Metodos.getClasificacion(user2);
		assertTrue(datos[0].equals("1"));
		assertTrue(datos[1].equals("0"));
		assertTrue(datos[2].equals("1"));
		assertTrue(datos[3].equals("0"));
		assertTrue(datos[4].equals("0"));
		
		ultimoID = Metodos.getUltimoId();
		this.partidaInserted.add(""+(ultimoID+1));
		t = new Tablero(ultimoID+1, user1, user2);
		t.abandono('b');
		t.guardarEnBD();
		
		datos = Metodos.getClasificacion(user1);
		assertTrue(datos[0].equals("2"));
		assertTrue(datos[1].equals("1"));
		assertTrue(datos[2].equals("1"));
		assertTrue(datos[3].equals("0"));
		assertTrue(datos[4].equals("3"));
		
		datos = Metodos.getClasificacion(user2);
		assertTrue(datos[0].equals("2"));
		assertTrue(datos[1].equals("1"));
		assertTrue(datos[2].equals("1"));
		assertTrue(datos[3].equals("0"));
		assertTrue(datos[4].equals("3"));
		
		ultimoID = Metodos.getUltimoId();
		this.partidaInserted.add(""+(ultimoID+1));
		t = new Tablero(ultimoID+1, user1, user2);
		t.guardarEnBD();
		
		datos = Metodos.getClasificacion(user1);
		assertTrue(datos[0].equals("3"));
		assertTrue(datos[1].equals("1"));
		assertTrue(datos[2].equals("1"));
		assertTrue(datos[3].equals("1"));
		assertTrue(datos[4].equals("4"));
		
		datos = Metodos.getClasificacion(user2);
		assertTrue(datos[0].equals("3"));
		assertTrue(datos[1].equals("1"));
		assertTrue(datos[2].equals("1"));
		assertTrue(datos[3].equals("1"));
		assertTrue(datos[4].equals("4"));
		
	}
		
}
